function untarget_by_func(entity, target_checker_func, log_if_triggered)
	local suppressed = false
	local AIComponents = EntityGetComponent(entity, "AnimalAIComponent")
	assert(type(AIComponents) == "table" and #AIComponents > 0, "Entity does not have AnimalAIComponents")
	
	for _, component in pairs(AIComponents) do
		local prey = tonumber(ComponentGetValue(component, "mGreatestPrey"))
		local do_suppression, extra_info = target_checker_func(prey, component)
		if do_suppression then
			ComponentSetValue(component, "mGreatestPrey", "0")
			suppressed = true
			if log_if_triggered then
				print("[ET Untargeter] Suppressed entity " .. entity .. " from trying to target entity " .. prey .. "! (" .. tostring(extra_info) .. ")")
			elseif extra_info then
				print("[ET Untargeter] Did not suppress entity " .. entity .. " from trying to target entity " .. prey .. "! (" .. tostring(extra_info) .. ")")
			end
		end
	end
	
	return suppressed
end

function untarget_player(entity, log_if_triggered)
	--[[
	local suppressed = false
	local AIComponents = EntityGetComponent(entity, "AnimalAIComponent")
	assert(type(AIComponents) == "table" and #AIComponents > 0, "Entity does not have AnimalAIComponents")
	
	for _, component in pairs(AIComponents) do
		local prey = tonumber(ComponentGetValue(component, "mGreatestPrey"))
		if IsPlayer(prey) then
			ComponentSetValue(component, "mGreatestPrey", "0")
			suppressed = true
			if log_if_triggered then
				print("[ET Untargeter] Suppressed entity " .. entity .. " from trying to target player " .. prey .. "!")
			end
		end
	end
	]]--
	
	return untarget_by_func(entity, function(target) return IsPlayer(target), "untarget_player" end, log_if_triggered)
end

function untarget_by_id(entity, target_id, log_if_triggered)
	--[[
	local suppressed = false
	local AIComponents = EntityGetComponent(entity, "AnimalAIComponent")
	assert(type(AIComponents) == "table" and #AIComponents > 0, "Entity does not have AnimalAIComponents")
	
	for _, component in pairs(AIComponents) do
		local prey = tonumber(ComponentGetValue(component, "mGreatestPrey"))
		if prey == tonumber(target_id) then
			ComponentSetValue(component, "mGreatestPrey", "0")
			suppressed = true
			if log_if_triggered then
				print("[ET Untargeter] Suppressed entity " .. entity .. " from trying to target entity " .. target_id .. "!")
			end
		end
	end
	]]--
	
	return untarget_by_func(entity, function(target) return target == tonumber(target_id), "untarget_by_id" end, log_if_triggered)
end

function untarget_all(entity, log_if_triggered)
	return untarget_by_func(entity, function(target) return target ~= 0, "untarget_all" end, log_if_triggered)
end
