-- all functions below are optional and can be left out

--[[

function OnModPreInit()
	print("Mod - OnModPreInit()") -- First this is called for all mods
end

function OnModInit()
	print("Mod - OnModInit()") -- After that this is called for all mods
end

function OnModPostInit()
	print("Mod - OnModPostInit()") -- Then this is called for all mods
end

function OnPlayerSpawned( player_entity ) -- This runs when player entity has been created
end

function OnWorldInitialized() -- This is called once the game world is initialized. Doesn't ensure any world chunks actually exist. Use OnPlayerSpawned to ensure the chunks around player have been loaded or created.
	GamePrint( "OnWorldInitialized() " .. tostring(GameGetFrameNum()) )
end

function OnWorldPreUpdate() -- This is called every time the game is about to start updating the world
	GamePrint( "Pre-update hook " .. tostring(GameGetFrameNum()) )
end

function OnWorldPostUpdate() -- This is called every time the game has finished updating the world
	GamePrint( "Post-update hook " .. tostring(GameGetFrameNum()) )
end

]]--

-- This code runs when all mods' filesystems are registered
-- ModLuaFileAppend( "data/scripts/gun/gun_actions.lua", "mods/example/files/actions.lua" ) -- Basically dofile("mods/example/files/actions.lua") will appear at the end of gun_actions.lua
-- ModMagicNumbersFileAdd( "mods/example/files/magic_numbers.xml" ) -- Will override some magic numbers using the specified file
-- ModRegisterAudioEventMappings( "mods/example/files/audio_events.txt" ) -- Use this to register custom fmod events. Event mapping files can be generated via File -> Export GUIDs in FMOD Studio.
-- ModMaterialsFileAdd( "mods/example/files/materials_rainbow.xml" ) -- Adds a new 'rainbow' material to materials

--print("Example mod init done")

local accum_prints = {}
local accum_gameprints = {}
local old_print = print
local old_gameprint = GamePrint

function print(...)
	local t = {...}
	local accum_value = ""
	for _, value in pairs(t) do
		accum_value = accum_value .. tostring(value) .. '\t'
	end
	
	table.insert(accum_prints, accum_value)
	
	old_print(...)
end

function GamePrint(value)
	table.insert(accum_gameprints, value)
	old_gameprint(value)
end

function OnPlayerSpawned( player_entity ) -- This runs when player entity has been created
	-- GamePrint("Happy mutating! :)")
	print = old_print
	GamePrint = old_gameprint
	
	print("[ET] BEGIN ACCUMULATED PRINT STATEMENTS:")
	for _, value in pairs(accum_prints) do
		print(value)
	end
	print("[ET] END ACCUMULATED PRINT STATEMENTS.")
	
	for _, value in pairs(accum_gameprints) do
		GamePrint(value)
	end
end

dofile_once("mods/enemy_traits/ET_CONFIG.lua")

local lua_appends = {
	["data/scripts/gun/gun_actions.lua"] = 
	{
		"mods/enemy_traits/enemy_traits_files/spells/radialshot.lua",
		"mods/enemy_traits/enemy_traits_files/spells/spiralshot.lua",
		"mods/enemy_traits/enemy_traits_files/spells/pathfinding.lua",
		"mods/enemy_traits/enemy_traits_files/spells/polybomb.lua",
		"mods/enemy_traits/enemy_traits_files/spells/healdrone.lua",
		"mods/enemy_traits/enemy_traits_files/spells/rocket_tiny.lua",
		"mods/enemy_traits/enemy_traits_files/spells/slimetrail.lua",
		"mods/enemy_traits/enemy_traits_files/spells/lasersight.lua",
		"mods/enemy_traits/enemy_traits_files/spells/rocket_tiny_timer.lua"
	},
	
	["data/scripts/perks/perk_list.lua"] = 
	{
		"mods/enemy_traits/enemy_traits_files/perks/mimic.lua",
		"mods/enemy_traits/enemy_traits_files/perks/bleed_gunpowder.lua",
		"mods/enemy_traits/enemy_traits_files/perks/wraith_retaliation.lua",
		"mods/enemy_traits/enemy_traits_files/perks/spiderlegs.lua",
		"mods/enemy_traits/enemy_traits_files/perks/wand_ally.lua",
		"mods/enemy_traits/enemy_traits_files/perks/crystal_minions.lua"
	},
	
	["data/scripts/items/drop_money.lua"] = 
	{
		"mods/enemy_traits/enemy_traits_files/scripts/loot_checks.lua"
	}
}

print("[Enemy Traits] Performing file appends...")

local total = 0
local count = 0
local done = 0
local extra = 0

for _, sources in pairs(lua_appends) do
	for _, source in pairs(sources) do
		total = total + 1
		local source_name = string.match(source, ".+/(.+)%..+$")
		extra = math.max(extra, string.len(source_name))
	end
end

print("[Enemy Traits] Found " .. total .. " sources.")

local padding = string.len(tostring(total))

for target, paths in pairs(lua_appends) do
	for _, source in ipairs(paths) do
		local target_name = string.match(target, ".+/(.+)%..+$")
		local source_name = string.match(source, ".+/(.+)%..+$")
		local source_length = string.len(source_name)
		local extra_padding = extra - source_length
		count = count + 1
		
		if ET_CONFIG[source_name] then
			ModLuaFileAppend(target, source)
			done = done + 1
			
			print(string.format("[Enemy Traits] [%0" .. padding .. "d/%d] Appended '%s'%" .. extra_padding .. "s to '%s'.", count, total, source_name, "", target_name))
		else
			print(string.format("[Enemy Traits] Skipped appending '%s' to '%s': Disabled via config.", source_name, target_name))
		end
	end
end

local append_summary = string.format("[Enemy Traits] Performed %s/%s appends. (%d%%)", done, total, done / total * 100)
print(append_summary)
GamePrint(append_summary)
